package cdcsignal_public;
import org.opensourcephysics.numerics.*;
import java.util.Random;

public class CdcODE implements ODE {
	  double l0, ln, V, tau, Abar; 
	  int n;
	  int historyLength;
	  double historyDt;
	  double epsilon;
	  int hillCoefficient;
	  double dVdt;
	  double Vcur;
	  double kminus;
	  double noise;
	  double lp_tip2;
	  double km_tip2;
	  Random r = new Random();
	  double[] state = new double[3];
	  double[][] stateHistory;
	  public double[] currentrate = new double[2];

	  public double[] getState() {
		    return state;
	  }
	  
	  public void setHistory(double dt, double ConeTau, double CtwoTau) {
		  stateHistory = new double[(int) (tau / dt)][3];
		  double t = state[2];
		  for(double[] s : stateHistory) {
			  s[0] = state[0]- ((ConeTau - state[0]) * (dt / tau));
			  s[1] = state[1]- ((CtwoTau - state[1]) * (dt / tau));
			  s[2] = t;
			  t -= dt;
		  }
	  }
	  
	  private double oldState(int i) {
		  return stateHistory[stateHistory.length-1][i];
	  }

	  public void advanceHistory() {
		  for(int j=stateHistory.length - 1 ; j > 0  ; j--) {
			  stateHistory[j]=stateHistory[j-1].clone();
		  }
		  stateHistory[0] = state;
	  }
	  
	  private double hill(double x, double K, int h) {
		  double xtoh = Math.pow(x, h);
		  return xtoh / (Math.pow(K,h) + xtoh);
	  }
	  
	  public void getRate(double[] state, double[] rate) {
		  int[] stateList = {0,1};
		  Vcur = V + dVdt * state[2];
		  double kpluscur, kminuscur;
		  for(int i : stateList) {  
			  kpluscur = (l0+ln*Math.pow(state[i]*Vcur,n))*Math.exp(-state[i]*Vcur)*(1/Vcur);
			  kminuscur = kminus * ((1 - (epsilon / 2)) + epsilon * hill(oldState(i),state[i],hillCoefficient));
			  if(i==1){
				  kpluscur *= lp_tip2;
				  kminuscur *= km_tip2;
			  }  
			  rate[i] = Math.max(noise * r.nextGaussian() + kpluscur, 0)*(1-state[0]-state[1]) 
			  - Math.max(noise * r.nextGaussian() + kminuscur, 0) * state[i];
			  currentrate[i] = rate[i];
		  }
		  rate[2] = 1;
	  }
}
