package cdcsignal_public;
import java.util.Random;

import org.opensourcephysics.controls.*;
import org.opensourcephysics.frames.*;
import org.opensourcephysics.numerics.ODESolver;
import org.opensourcephysics.numerics.Verlet;

public class CdcODEwithCalcs_App extends AbstractSimulation {
	CdcODE cdcODE = new CdcODE();
	ODESolver solver = new Verlet(cdcODE);
	PlotFrame frame = new PlotFrame("time","Ctip1 over Ctot and Ctip2 over Ctot","Cdc42 ODE Model");
	int step_number;
	int record_every;
	double measurementNoise;
	Random r = new Random();
	
	public void initialize() {
	    double Cone = control.getDouble("initial tip1");
	    double Ctwo = control.getDouble("initial tip2");
	    double ConeTau = control.getDouble("tip1 tau ago");
	    double CtwoTau = control.getDouble("tip2 tau ago");
	    double l0 = control.getDouble("l0");
	    double ln = control.getDouble("ln");
	    int n = control.getInt("n");
	    double kminus = control.getDouble("kminus");
	    double V = control.getDouble("Ctotbar");
	    double tau = control.getDouble("tau");
	    double epsilon = control.getDouble("epsilon");
	    double dVdt = control.getDouble("dCtotbar dt");
	    int hillCoefficient = control.getInt("hill coefficient");
	    double noise = control.getDouble("noise");
	    measurementNoise = control.getDouble("measurement noise level");
	    double dt = control.getDouble("dt");
	    int recordEvery = control.getInt("record every...");
	    double lp_tip2 = control.getDouble("multiplier for tip2 on rate");
	    double km_tip2 = control.getDouble("multiplier for tip2 off rate");
	    r = new Random();
	    setDelayTime(0);
	    
	    step_number = 0;
	    record_every = recordEvery;
	    cdcODE.state[2] = 0;
	    cdcODE.l0 = l0;
	    cdcODE.ln = ln;
	    cdcODE.V = V;
	    cdcODE.n = n;
	    cdcODE.dVdt = dVdt;
	    cdcODE.tau = tau;
	    cdcODE.epsilon = epsilon;
	    cdcODE.hillCoefficient = hillCoefficient;
	    cdcODE.kminus = kminus;
	    cdcODE.state[0] = Cone;
	    cdcODE.state[1] = Ctwo;
	    cdcODE.lp_tip2 = lp_tip2;
	    cdcODE.km_tip2 = km_tip2;
	    cdcODE.noise = noise / Math.sqrt(dt);
	    cdcODE.setHistory(dt,ConeTau,CtwoTau);
	    solver.setStepSize(dt);
	}
	
	public void doStep() {
		if(step_number % record_every == 0) {
		    double[] currentPoint = {cdcODE.state[0]* (1+measurementNoise*r.nextGaussian()),
		    		cdcODE.state[1]* (1+measurementNoise*r.nextGaussian())};
		    for(int i=0;i<2;i++) {
		    	frame.append(i, cdcODE.state[2], currentPoint[i]); 
		    }
		}
	    
	    solver.step(); 
	    step_number++;  
	    cdcODE.advanceHistory();
    }
	
	public void reset() {
		control.setValue("l0",9);
		control.setValue("ln",25.86848);
		control.setValue("n",4);
		control.setValue("kminus",4);
		control.setValue("Ctotbar",6.5);
		control.setValue("dCtotbar dt",0.0271);
		control.setValue("tau", 2);
		control.setValue("epsilon",.5375);
		control.setValue("hill coefficient",40);
		control.setValue("noise",0);
		control.setValue("initial tip1", 0.02);
		control.setValue("initial tip2", 0.7);
		control.setValue("tip1 tau ago", 0.02);
	    control.setValue("tip2 tau ago", 0.7);
		control.setValue("dt", 0.05);
		control.setValue("record every...",10);
		control.setValue("multiplier for tip2 on rate",1);
		control.setValue("multiplier for tip2 off rate",1);
		control.setValue("measurement noise level", 0);
	}
	
	public static void main(String[] args) {
		SimulationControl.createApp(new CdcODEwithCalcs_App());
	}
}
